#ifdef BEFORE_PARSE
#undef BEFORE_PARSE
CPUGroup.add_option("-c", "--core")
	.dest("Core")
	.choices({
		"irint",
		"irjit",
		"host",
	})
	.help(
		"Which CPU core to use\n"
		"host only exists on x86_64\n"
		"[irint, irjit, host]\n"
	)
	.set_default("irjit");

CPUGroup.add_option("-m", "--multiblock")
	.action("store_true")
	.dest("Multiblock")
	.help(
		"Controls multiblock code compilation\n"
		"Can cause long JIT compilation times and stutter\n"
	)
	.set_default(false);

CPUGroup.add_option("--no-multiblock")
	.action("store_false")
	.dest("Multiblock");

CPUGroup.add_option("-n", "--maxinst")
	.dest("MaxInst")
	.help(
		"Maximum number of instruction to store in a block"
	)
	.set_default(5000);

CPUGroup.add_option("-T", "--threads")
	.dest("Threads")
	.help(
		"Number of physical hardware threads to tell the process we have.\n"
		"0 will auto detect.\n"
	)
	.set_default(0);

CPUGroup.add_option("--cacheobjectcodecompilation")
	.dest("CacheObjectCodeCompilation")
	.choices({
		"none",
		"read",
		"readwrite",
	})
	.help(
		"Cache JIT object code to drive.\n"
		"Allows JIT code to be shared between applications\n"
	)
	.set_default("none");

CPUGroup.add_option("--enableavx")
	.action("store_true")
	.dest("EnableAVX")
	.help(
		"Determines whether or not we use the expanded register file for AVX or not"
	)
	.set_default(false);

CPUGroup.add_option("--no-enableavx")
	.action("store_false")
	.dest("EnableAVX");


EmulationGroup.add_option("-R", "--rootfs")
	.dest("RootFS")
	.help(
		"Which Root filesystem prefix to use\n"
		"This can be a filesystem path\n"
		"	eg: ~/RootFS/Debian_x86_64\n"
		"Or this can be a name of a rootfs\n"
		"If the named rootfs exists in the FEX data folder then it will use that one\n"
		"	eg: $HOME/.fex-emu/RootFS/<RootFS name>/\n"
		"Or if you have XDG_DATA_HOME the config will search in that directory\n"
		"	eg: $XDG_DATA_HOME/.fex-emu/RootFS/<RootFS name>/\n"
	)
	.set_default("");

EmulationGroup.add_option("-t", "--thunkhostlibs")
	.dest("ThunkHostLibs")
	.help(
		"Folder to find the host-side thunking libraries."
	)
	.set_default("/usr/lib/fex-emu/HostThunks/");

EmulationGroup.add_option("-j", "--thunkguestlibs")
	.dest("ThunkGuestLibs")
	.help(
		"Folder to find the guest-side thunking libraries."
	)
	.set_default("/usr/share/fex-emu/GuestThunks/");

EmulationGroup.add_option("--thunkhostlibs32")
	.dest("ThunkHostLibs32")
	.help(
		"Folder to find the 32-bit host-side thunking libraries."
	)
	.set_default("/usr/lib/fex-emu/HostThunks_32/");

EmulationGroup.add_option("--thunkguestlibs32")
	.dest("ThunkGuestLibs32")
	.help(
		"Folder to find the 32-bit guest-side thunking libraries."
	)
	.set_default("/usr/share/fex-emu/GuestThunks_32/");

EmulationGroup.add_option("-k", "--thunkconfig")
	.dest("ThunkConfig")
	.help(
		"A json file specifying where to overlay the thunks.\n"
		"This can be a filesystem path\n"
		"	eg: ~/MyThunkConfig.json\n"
		"Or this can be a named of a Thunk config file\n"
		"If the named config file exists in the FEX data folder folder the it will use that one\n"
		"	eg: $HOME/.fex-emu/ThunkConfigs/<ThunkConfig name>\n"
		"Or if you have XDG_DATA_HOME the config will search in that directory\n"
		"	eg: $XDG_DATA_HOME/.fex-emu/ThunkConfigs/<ThunkConfig name>\n"
	)
	.set_default("");

EmulationGroup.add_option("-E", "--env")
	.dest("Env")
	.help(
		"Adds an environment variable to the emulated environment."
	)
	.set_default("");

EmulationGroup.add_option("-H", "--hostenv")
	.dest("HostEnv")
	.help(
		"Adds an environment variable to the host environment.\n"
		"This can be useful for setting environment variables that thunks can pick up.\n"
		"Typically isn't necessary since the guest libc isn't thunked. But is possible.\n"
	)
	.set_default("");

EmulationGroup.add_option("--additionalarguments")
	.dest("AdditionalArguments")
	.help(
		"Allows the user to pass additional arguments to the application"
	)
	.set_default("");


DebugGroup.add_option("-S", "--singlestep")
	.action("store_true")
	.dest("SingleStep")
	.help(
		"Single stepping configuration."
	)
	.set_default(false);

DebugGroup.add_option("--no-singlestep")
	.action("store_false")
	.dest("SingleStep");

DebugGroup.add_option("-G", "--gdbserver")
	.action("store_true")
	.dest("GdbServer")
	.help(
		"Enables the GDB server."
	)
	.set_default(false);

DebugGroup.add_option("--no-gdbserver")
	.action("store_false")
	.dest("GdbServer");

DebugGroup.add_option("--dumpir")
	.dest("DumpIR")
	.help(
		"Folder to dump the IR in to.\n"
		"[no, stdout, stderr, <Folder>]\n"
	)
	.set_default("no");

DebugGroup.add_option("-g", "--dumpgprs")
	.action("store_true")
	.dest("DumpGPRs")
	.help(
		"When the test harness ends, print the GPR state."
	)
	.set_default(false);

DebugGroup.add_option("--no-dumpgprs")
	.action("store_false")
	.dest("DumpGPRs");

DebugGroup.add_option("-O0", "--o0")
	.action("store_true")
	.dest("O0")
	.help(
		"Disables optimizations passes for debugging."
	)
	.set_default(false);

DebugGroup.add_option("--no-o0")
	.action("store_false")
	.dest("O0");

DebugGroup.add_option("--sra")
	.action("store_true")
	.dest("SRA")
	.help(
		"Set to false to disable Static Register Allocation"
	)
	.set_default(true);

DebugGroup.add_option("--no-sra")
	.action("store_false")
	.dest("SRA");

DebugGroup.add_option("--force32bitallocator")
	.action("store_true")
	.dest("Force32BitAllocator")
	.help(
		"Forces use of the 32-bit allocator on 32-bit applications\n"
		"Used to work around ulimit problems of CI runner\n"
		"Potentially useful for debugging memory problems\n"
		"32-bit allocator is always used if your host kernel is older than 4.17\n"
	)
	.set_default(false);

DebugGroup.add_option("--no-force32bitallocator")
	.action("store_false")
	.dest("Force32BitAllocator");

DebugGroup.add_option("--globaljitnaming")
	.action("store_true")
	.dest("GlobalJITNaming")
	.help(
		"Uses JITSymbols to name all JIT state as one symbol\n"
		"Useful for querying how much time is spent inside of the JIT\n"
		"Profiling tools will show JIT time as FEXJIT\n"
	)
	.set_default(false);

DebugGroup.add_option("--no-globaljitnaming")
	.action("store_false")
	.dest("GlobalJITNaming");

DebugGroup.add_option("--libraryjitnaming")
	.action("store_true")
	.dest("LibraryJITNaming")
	.help(
		"Uses JITSymbols to name JIT symbols grouped by library\n"
		"Useful for querying how much time is spent in each guest library\n"
		"Can be used to help guide thunk generation\n"
	)
	.set_default(false);

DebugGroup.add_option("--no-libraryjitnaming")
	.action("store_false")
	.dest("LibraryJITNaming");

DebugGroup.add_option("--blockjitnaming")
	.action("store_true")
	.dest("BlockJITNaming")
	.help(
		"Uses JITSymbols to name JIT symbols\n"
		"Useful for determining hot blocks of code\n"
		"Has some file writing overhead per JIT block\n"
	)
	.set_default(false);

DebugGroup.add_option("--no-blockjitnaming")
	.action("store_false")
	.dest("BlockJITNaming");

DebugGroup.add_option("--gdbsymbols")
	.action("store_true")
	.dest("GDBSymbols")
	.help(
		"Integrates with GDB using the JIT interface.\n"
		"Needs the fex jit loader in GDB, which can be loaded via `jit-reader-load libFEXGDBReader.so.`\n"
		"Also needs x86_64-linux-gnu-objdump in PATH.\n"
		"Can be very slow.\n"
	)
	.set_default(false);

DebugGroup.add_option("--no-gdbsymbols")
	.action("store_false")
	.dest("GDBSymbols");

DebugGroup.add_option("--injectlibsegfault")
	.action("store_true")
	.dest("InjectLibSegFault")
	.help(
		"Sets the environment variable LD_PRELOAD=libSegFault.so\n"
		"This allows the user to very easily enable libSegFault without dealing with environment variables\n"
		"Very useful for applications that have launch scripts that set the variable to nothing at launch\n"
		"Set this in an application configuration for injecting in to only specific applications.\n"
		"	Note: If x86/x86_64 libSegFault.so isn't installed then this option won't work.\n"
	)
	.set_default(false);

DebugGroup.add_option("--no-injectlibsegfault")
	.action("store_false")
	.dest("InjectLibSegFault");


LoggingGroup.add_option("-s", "--silentlog")
	.action("store_true")
	.dest("SilentLog")
	.help(
		"Disables logging"
	)
	.set_default(true);

LoggingGroup.add_option("--no-silentlog")
	.action("store_false")
	.dest("SilentLog");

LoggingGroup.add_option("-o", "--outputlog")
	.dest("OutputLog")
	.help(
		"File to write FEX output to.\n"
		"[stdout, stderr, server, <Filename>]\n"
	)
	.set_default("server");


HacksGroup.add_option("--smcchecks")
	.dest("SMCChecks")
	.help(
		"Checks code for modification before execution.\n"
		"	none: No checks\n"
		"	mtrack: Page tracking based invalidation\n"
		"	full: Validate code before every run (slow)\n"
		"	mman: Invalidate on mmap, mprotect, munmap (deprecated, use mtrack)\n"
	)
	.set_default("mtrack");

HacksGroup.add_option("--tsoenabled")
	.action("store_true")
	.dest("TSOEnabled")
	.help(
		"Controls TSO IR ops.\n"
		"Highly likely to break any multithreaded application if disabled.\n"
	)
	.set_default(true);

HacksGroup.add_option("--no-tsoenabled")
	.action("store_false")
	.dest("TSOEnabled");

HacksGroup.add_option("--tsoautomigration")
	.action("store_true")
	.dest("TSOAutoMigration")
	.help(
		"Automatically enables TSO when shared memory is used.\n"
		"Should work without issues in most cases.\n"
	)
	.set_default(true);

HacksGroup.add_option("--no-tsoautomigration")
	.action("store_false")
	.dest("TSOAutoMigration");

HacksGroup.add_option("--x87reducedprecision")
	.action("store_true")
	.dest("X87ReducedPrecision")
	.help(
		"Emulates X87 floating point using 64-bit precision. This reduces emulation accuracy and may result in rendering bugs."
	)
	.set_default(false);

HacksGroup.add_option("--no-x87reducedprecision")
	.action("store_false")
	.dest("X87ReducedPrecision");

HacksGroup.add_option("--abilocalflags")
	.action("store_true")
	.dest("ABILocalFlags")
	.help(
		"When enabled enables an optimization around flags.\n"
		"Assumes flags are not used across cals.\n"
		"Hand-written assembly can violate this assumption.\n"
	)
	.set_default(false);

HacksGroup.add_option("--no-abilocalflags")
	.action("store_false")
	.dest("ABILocalFlags");

HacksGroup.add_option("--abinopf")
	.action("store_true")
	.dest("ABINoPF")
	.help(
		"When enabled enables an optimization around parity flag calculation.\n"
		"Removes the calculation of the parity flag from GPR instructions.\n"
		"Assuming no uses rely on it\n"
	)
	.set_default(false);

HacksGroup.add_option("--no-abinopf")
	.action("store_false")
	.dest("ABINoPF");

HacksGroup.add_option("--paranoidtso")
	.action("store_true")
	.dest("ParanoidTSO")
	.help(
		"Makes TSO operations even more strict.\n"
		"Forces vector loadstores to also become atomic.\n"
	)
	.set_default(false);

HacksGroup.add_option("--no-paranoidtso")
	.action("store_false")
	.dest("ParanoidTSO");

HacksGroup.add_option("--stallprocess")
	.action("store_true")
	.dest("StallProcess")
	.help(
		"Forces a process to stall out on initialization\n"
		"Useful for a process that keeps restarting and doesn't work\n"
	)
	.set_default(false);

HacksGroup.add_option("--no-stallprocess")
	.action("store_false")
	.dest("StallProcess");

HacksGroup.add_option("--hidehypervisorbit")
	.action("store_true")
	.dest("HideHypervisorBit")
	.help(
		"Hides the hypervisor CPUID bit when set.\n"
		"Should only be used for applications that have issues with this set.\n"
	)
	.set_default(false);

HacksGroup.add_option("--no-hidehypervisorbit")
	.action("store_false")
	.dest("HideHypervisorBit");


MiscGroup.add_option("--aotircapture")
	.action("store_true")
	.dest("AOTIRCapture")
	.help(
		"Captures IR and generates an AOT IR cache.\n"
		"Captures both the loaded executable and libraries it loads.\n"
	)
	.set_default(false);

MiscGroup.add_option("--no-aotircapture")
	.action("store_false")
	.dest("AOTIRCapture");

MiscGroup.add_option("--aotirgenerate")
	.action("store_true")
	.dest("AOTIRGenerate")
	.help(
		"Scans file for executable code and generates an AOT IR cache.\n"
		"Does not run the executable.\n"
	)
	.set_default(false);

MiscGroup.add_option("--no-aotirgenerate")
	.action("store_false")
	.dest("AOTIRGenerate");

MiscGroup.add_option("--aotirload")
	.action("store_true")
	.dest("AOTIRLoad")
	.help(
		"Loads an AOT IR cache for the loaded executable."
	)
	.set_default(false);

MiscGroup.add_option("--no-aotirload")
	.action("store_false")
	.dest("AOTIRLoad");

MiscGroup.add_option("--serversocketpath")
	.dest("ServerSocketPath")
	.help(
		"Override for a FEXServer socket path. Only useful for chroots."
	)
	.set_default("");


#endif
#ifdef AFTER_PARSE
#undef AFTER_PARSE
if (Options.is_set_by_user("Core")) {
	fextl::string UserValue = Options["Core"];
	Set(FEXCore::Config::ConfigOption::CONFIG_CORE, FEXCore::Config::Handler::CoreHandler(UserValue));
}
if (Options.is_set_by_user("Multiblock")) {
	bool UserValue = Options.get("Multiblock");
	Set(FEXCore::Config::ConfigOption::CONFIG_MULTIBLOCK, fextl::fmt::format("{:d}", UserValue));
}
if (Options.is_set_by_user("MaxInst")) {
	int32 UserValue = Options.get("MaxInst");
	Set(FEXCore::Config::ConfigOption::CONFIG_MAXINST, fextl::fmt::format("{}", UserValue));
}
if (Options.is_set_by_user("Threads")) {
	uint32 UserValue = Options.get("Threads");
	Set(FEXCore::Config::ConfigOption::CONFIG_THREADS, fextl::fmt::format("{}", UserValue));
}
if (Options.is_set_by_user("CacheObjectCodeCompilation")) {
	fextl::string UserValue = Options["CacheObjectCodeCompilation"];
	Set(FEXCore::Config::ConfigOption::CONFIG_CACHEOBJECTCODECOMPILATION, FEXCore::Config::Handler::CacheObjectCodeHandler(UserValue));
}
if (Options.is_set_by_user("EnableAVX")) {
	bool UserValue = Options.get("EnableAVX");
	Set(FEXCore::Config::ConfigOption::CONFIG_ENABLEAVX, fextl::fmt::format("{:d}", UserValue));
}
if (Options.is_set_by_user("RootFS")) {
	fextl::string UserValue = Options["RootFS"];
	Set(FEXCore::Config::ConfigOption::CONFIG_ROOTFS, (UserValue));
}
if (Options.is_set_by_user("ThunkHostLibs")) {
	fextl::string UserValue = Options["ThunkHostLibs"];
	Set(FEXCore::Config::ConfigOption::CONFIG_THUNKHOSTLIBS, (UserValue));
}
if (Options.is_set_by_user("ThunkGuestLibs")) {
	fextl::string UserValue = Options["ThunkGuestLibs"];
	Set(FEXCore::Config::ConfigOption::CONFIG_THUNKGUESTLIBS, (UserValue));
}
if (Options.is_set_by_user("ThunkHostLibs32")) {
	fextl::string UserValue = Options["ThunkHostLibs32"];
	Set(FEXCore::Config::ConfigOption::CONFIG_THUNKHOSTLIBS32, (UserValue));
}
if (Options.is_set_by_user("ThunkGuestLibs32")) {
	fextl::string UserValue = Options["ThunkGuestLibs32"];
	Set(FEXCore::Config::ConfigOption::CONFIG_THUNKGUESTLIBS32, (UserValue));
}
if (Options.is_set_by_user("ThunkConfig")) {
	fextl::string UserValue = Options["ThunkConfig"];
	Set(FEXCore::Config::ConfigOption::CONFIG_THUNKCONFIG, (UserValue));
}
if (Options.is_set_by_user("Env")) {
	auto Array = Options.all("Env");
	for (auto iter = Array.begin(); iter != Array.end(); ++iter) {
		Set(FEXCore::Config::ConfigOption::CONFIG_ENV, *iter);
	}
}
if (Options.is_set_by_user("HostEnv")) {
	auto Array = Options.all("HostEnv");
	for (auto iter = Array.begin(); iter != Array.end(); ++iter) {
		Set(FEXCore::Config::ConfigOption::CONFIG_HOSTENV, *iter);
	}
}
if (Options.is_set_by_user("AdditionalArguments")) {
	auto Array = Options.all("AdditionalArguments");
	for (auto iter = Array.begin(); iter != Array.end(); ++iter) {
		Set(FEXCore::Config::ConfigOption::CONFIG_ADDITIONALARGUMENTS, *iter);
	}
}
if (Options.is_set_by_user("SingleStep")) {
	bool UserValue = Options.get("SingleStep");
	Set(FEXCore::Config::ConfigOption::CONFIG_SINGLESTEP, fextl::fmt::format("{:d}", UserValue));
}
if (Options.is_set_by_user("GdbServer")) {
	bool UserValue = Options.get("GdbServer");
	Set(FEXCore::Config::ConfigOption::CONFIG_GDBSERVER, fextl::fmt::format("{:d}", UserValue));
}
if (Options.is_set_by_user("DumpIR")) {
	fextl::string UserValue = Options["DumpIR"];
	Set(FEXCore::Config::ConfigOption::CONFIG_DUMPIR, (UserValue));
}
if (Options.is_set_by_user("DumpGPRs")) {
	bool UserValue = Options.get("DumpGPRs");
	Set(FEXCore::Config::ConfigOption::CONFIG_DUMPGPRS, fextl::fmt::format("{:d}", UserValue));
}
if (Options.is_set_by_user("O0")) {
	bool UserValue = Options.get("O0");
	Set(FEXCore::Config::ConfigOption::CONFIG_O0, fextl::fmt::format("{:d}", UserValue));
}
if (Options.is_set_by_user("SRA")) {
	bool UserValue = Options.get("SRA");
	Set(FEXCore::Config::ConfigOption::CONFIG_SRA, fextl::fmt::format("{:d}", UserValue));
}
if (Options.is_set_by_user("Force32BitAllocator")) {
	bool UserValue = Options.get("Force32BitAllocator");
	Set(FEXCore::Config::ConfigOption::CONFIG_FORCE32BITALLOCATOR, fextl::fmt::format("{:d}", UserValue));
}
if (Options.is_set_by_user("GlobalJITNaming")) {
	bool UserValue = Options.get("GlobalJITNaming");
	Set(FEXCore::Config::ConfigOption::CONFIG_GLOBALJITNAMING, fextl::fmt::format("{:d}", UserValue));
}
if (Options.is_set_by_user("LibraryJITNaming")) {
	bool UserValue = Options.get("LibraryJITNaming");
	Set(FEXCore::Config::ConfigOption::CONFIG_LIBRARYJITNAMING, fextl::fmt::format("{:d}", UserValue));
}
if (Options.is_set_by_user("BlockJITNaming")) {
	bool UserValue = Options.get("BlockJITNaming");
	Set(FEXCore::Config::ConfigOption::CONFIG_BLOCKJITNAMING, fextl::fmt::format("{:d}", UserValue));
}
if (Options.is_set_by_user("GDBSymbols")) {
	bool UserValue = Options.get("GDBSymbols");
	Set(FEXCore::Config::ConfigOption::CONFIG_GDBSYMBOLS, fextl::fmt::format("{:d}", UserValue));
}
if (Options.is_set_by_user("InjectLibSegFault")) {
	bool UserValue = Options.get("InjectLibSegFault");
	Set(FEXCore::Config::ConfigOption::CONFIG_INJECTLIBSEGFAULT, fextl::fmt::format("{:d}", UserValue));
}
if (Options.is_set_by_user("SilentLog")) {
	bool UserValue = Options.get("SilentLog");
	Set(FEXCore::Config::ConfigOption::CONFIG_SILENTLOG, fextl::fmt::format("{:d}", UserValue));
}
if (Options.is_set_by_user("OutputLog")) {
	fextl::string UserValue = Options["OutputLog"];
	Set(FEXCore::Config::ConfigOption::CONFIG_OUTPUTLOG, (UserValue));
}
if (Options.is_set_by_user("SMCChecks")) {
	fextl::string UserValue = Options["SMCChecks"];
	Set(FEXCore::Config::ConfigOption::CONFIG_SMCCHECKS, FEXCore::Config::Handler::SMCCheckHandler(UserValue));
}
if (Options.is_set_by_user("TSOEnabled")) {
	bool UserValue = Options.get("TSOEnabled");
	Set(FEXCore::Config::ConfigOption::CONFIG_TSOENABLED, fextl::fmt::format("{:d}", UserValue));
}
if (Options.is_set_by_user("TSOAutoMigration")) {
	bool UserValue = Options.get("TSOAutoMigration");
	Set(FEXCore::Config::ConfigOption::CONFIG_TSOAUTOMIGRATION, fextl::fmt::format("{:d}", UserValue));
}
if (Options.is_set_by_user("X87ReducedPrecision")) {
	bool UserValue = Options.get("X87ReducedPrecision");
	Set(FEXCore::Config::ConfigOption::CONFIG_X87REDUCEDPRECISION, fextl::fmt::format("{:d}", UserValue));
}
if (Options.is_set_by_user("ABILocalFlags")) {
	bool UserValue = Options.get("ABILocalFlags");
	Set(FEXCore::Config::ConfigOption::CONFIG_ABILOCALFLAGS, fextl::fmt::format("{:d}", UserValue));
}
if (Options.is_set_by_user("ABINoPF")) {
	bool UserValue = Options.get("ABINoPF");
	Set(FEXCore::Config::ConfigOption::CONFIG_ABINOPF, fextl::fmt::format("{:d}", UserValue));
}
if (Options.is_set_by_user("ParanoidTSO")) {
	bool UserValue = Options.get("ParanoidTSO");
	Set(FEXCore::Config::ConfigOption::CONFIG_PARANOIDTSO, fextl::fmt::format("{:d}", UserValue));
}
if (Options.is_set_by_user("StallProcess")) {
	bool UserValue = Options.get("StallProcess");
	Set(FEXCore::Config::ConfigOption::CONFIG_STALLPROCESS, fextl::fmt::format("{:d}", UserValue));
}
if (Options.is_set_by_user("HideHypervisorBit")) {
	bool UserValue = Options.get("HideHypervisorBit");
	Set(FEXCore::Config::ConfigOption::CONFIG_HIDEHYPERVISORBIT, fextl::fmt::format("{:d}", UserValue));
}
if (Options.is_set_by_user("AOTIRCapture")) {
	bool UserValue = Options.get("AOTIRCapture");
	Set(FEXCore::Config::ConfigOption::CONFIG_AOTIRCAPTURE, fextl::fmt::format("{:d}", UserValue));
}
if (Options.is_set_by_user("AOTIRGenerate")) {
	bool UserValue = Options.get("AOTIRGenerate");
	Set(FEXCore::Config::ConfigOption::CONFIG_AOTIRGENERATE, fextl::fmt::format("{:d}", UserValue));
}
if (Options.is_set_by_user("AOTIRLoad")) {
	bool UserValue = Options.get("AOTIRLoad");
	Set(FEXCore::Config::ConfigOption::CONFIG_AOTIRLOAD, fextl::fmt::format("{:d}", UserValue));
}
if (Options.is_set_by_user("ServerSocketPath")) {
	fextl::string UserValue = Options["ServerSocketPath"];
	Set(FEXCore::Config::ConfigOption::CONFIG_SERVERSOCKETPATH, (UserValue));
}
#endif
#ifdef ENVLOADER
#undef ENVLOADER
if (false) {}
else if (Key == "FEX_CORE") {
Value = FEXCore::Config::Handler::CoreHandler(Value);
}
else if (Key == "FEX_CACHEOBJECTCODECOMPILATION") {
Value = FEXCore::Config::Handler::CacheObjectCodeHandler(Value);
}
else if (Key == "FEX_SMCCHECKS") {
Value = FEXCore::Config::Handler::SMCCheckHandler(Value);
}
#endif
